#include <LEDA/point.h>
#include <LEDA/queue.h>

/*****************************************************************************/

extern char interval_version[], main_version[], lbfs_version[];

/*****************************************************************************/

enum flies_status { FLIES_FLIES=0, FLIES_NEIGHBOUR=1, FLIES_OK=2 };

inline char *description (const flies_status status) {
	if (status == FLIES_FLIES) {
		return "flies";
	} else if (status == FLIES_NEIGHBOUR) {
		return "has a neighbour that flies";
	} else {
		return "is OK";
	}
}

inline ostream & operator << (ostream &out, const flies_status &status) {
	return out << description (status);
};

enum flies_choice { PICK_BETA=0, PICK_ALPHA=1 };

inline ostream & operator << (ostream &out, const flies_choice &choice) {
	if (choice == PICK_BETA) {
		return out << "beta";
	} else {
		return out << "alpha";
	}
};

typedef flies_choice flies_scheme[3][3];

bool flies (const labelled_graph&, const lexicographic_order&, const node);
flies_status get_flies_status (const labelled_graph&, const lexicographic_order&, const node);

extern flies_scheme flies_strong;
extern flies_scheme flies_weak;
extern flies_scheme flies_mixed;

/*****************************************************************************/

tiebreaker picklast;
tiebreaker picknonflier;

/*****************************************************************************/

const int MAX_SWEEPS = 5;
extern vertex_order sweep[MAX_SWEEPS];

/*****************************************************************************/

bool umbrella (const labelled_graph&, const vertex_order&);

typedef bool recognition_algorithm (labelled_graph&, node = NULL, tiebreaker = arbitrary, void* = NULL, void* = NULL);
recognition_algorithm four_plus;
recognition_algorithm four_star;
recognition_algorithm old;
recognition_algorithm three_sweep;
recognition_algorithm five_sweep;

/*****************************************************************************/

enum file_format { GraphEd, LEDA };

//@Man:
class options_t {
public:
	bool			verbose;
	bool			full_info;
	bool			debug;
	bool			edit;
	flies_scheme		flies;
	char			*flies_name;
	recognition_algorithm	*algorithm;
	int			num_sweeps;

	inline options_t () {
		verbose = edit = false;
		flies = flies_mixed;
		flies_name = "Mixed";
		algorithm = three_sweep;
		num_sweeps = 3;
		full_info = false;
	}
};

extern options_t options;

/*****************************************************************************/

//@Man:
class statistics_t {
private:
	node_array<unsigned long>	num_orders;
	node_array<unsigned long>	num_umbrella_orders;
	unsigned long			tb_usage[3][3];
	const labelled_graph		*the_graph;
	bool				found_result[2];
	queue<string>			reason_for_interest;
	bool				result_listed;
	bool				last_result;
	node_array<unsigned long>       num_mismatches;

public:
	inline void reset () {
		found_result[false] = false;
		found_result[true]  = false;
		result_listed = false;
	};
	inline void init (const labelled_graph &G) {
		num_orders.init (G, 0);
		num_umbrella_orders.init (G, 0);
		num_mismatches.init (G, 0);
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				tb_usage[i][j] = 0;
			}
		}
		the_graph = &G;
		reset ();
	};
	inline statistics_t (const labelled_graph &G) { init (G); };
	inline statistics_t () { };
	void count (node, bool);
	void count (const labelled_graph&, node, flies_status, node, flies_status);
	void count (node, const vertex_order&, const vertex_order&);
	inline bool interesting () {
		return !reason_for_interest.empty ();
	};
	void print_reason (ostream&);

	friend ostream & operator << (ostream&, const statistics_t&);
};

extern statistics_t statistics;
