#ifndef TIM_DEGREE_SEQUENCE_H
#define TIM_DEGREE_SEQUENCE_H

#include <LEDA/graph.h>
#include <LEDA/array.h>

typedef int degree_t;

/*****************************************************************************/
/* Degree sequence                                                           */
/*****************************************************************************/

class degree_sequence : public array<degree_t> {
	degree_t number_of_edges;
	degree_t total;
	degree_t excess;
	degree_t lower;
	degree_t upper;

public:
	degree_sequence (int n, degree_t m = 0, degree_t l = 0, degree_t u = -1) :
		array<degree_t> (n)
		{
			lower = l;
			upper = ((u < 0) || (u >= n)) ? (n-1) : u;
			number_of_edges = m;
			total = 2*m;
			excess = total - (n * lower);
		};
	degree_sequence (graph &);
	bool lexicographic_first ();
	bool lexicographic_next ();
	friend ostream & operator << (ostream &, degree_sequence &);
	bool is_graphical ();
	bool is_ktree_graphical (int = 2);
	friend bool operator == (degree_sequence &x, degree_sequence &y);
	friend bool operator != (degree_sequence &x, degree_sequence &y)
		{ return !(x == y); };
};

/*****************************************************************************/
/* Graph constructors                                                        */
/*****************************************************************************/

bool ktree (graph &, degree_sequence &, int = 2);

#endif
