#ifndef TIM_GRAPH_H
#define TIM_GRAPH_H

#include <fstream.h>
#include <LEDA/point.h>
#include <LEDA/graph.h>
#include <Tim/GA.h>

/*****************************************************************************/

//@Man:
class labelled_node {
public:
	point p;
	string label;

	//inline labelled_node (point p) : point (p) {};
	//inline point& set (const point &p) { return point::operator = (p); };
	//inline point& operator = (const point &p) { return set (p); };
	inline labelled_node () {};
	//inline bool operator == (const point &p) { return point::operator == (p); };
	//inline bool operator != (const point &p) { return point::operator != (p); };
	//friend ostream& operator << (ostream&, const labelled_node&);
	//friend istream& operator >> (istream&, const labelled_node&);
};

inline void Print (const labelled_node &v, ostream &out) {
	out << v.p << " " << v.label;
};
inline void Read (labelled_node &v, istream &in) {
	in >> v.p >> v.label;
};

/*****************************************************************************/

#if 0
//@Man:
class edge_info : public int {
};
#endif

/*****************************************************************************/
/* Read/write graphs in GraphEd format                                       */
/*****************************************************************************/

#if 0
int write_GAgraph (graph*, char*);
int write_GAgraph (graph*, ostream);

#ifdef GA_H
int read_GAgraph (GAGraph*, char*);
#endif
#endif

/*****************************************************************************/

//@Man:
class labelled_graph : public GRAPH<labelled_node,int> {
private:
	GAGraph GAG;
	void add_attributes ();

public:
	//node_array<string> label;
	int read_GraphEd (char*);
	int write_GraphEd (ostream);
	inline int write_GraphEd (char *filename) { return write_GraphEd (ofstream (filename)); };
	//int read (string);
	//int write (string);
        void edit (string);
	void build_labels ();
	inline labelled_graph () : GAG (this) { };
	~labelled_graph () {};
};

void graph_edit (labelled_graph&, bool, string);

/*****************************************************************************/
/* Graph generators                                                          */
/*****************************************************************************/

void petersen_graph (graph&);
void circulant (graph&, int, int);
void cycle (graph&, int);

/*****************************************************************************/
/* Graph searches                                                            */
/*****************************************************************************/

void MCS (graph&, node&, node_array<int>&);

/*****************************************************************************/
/* Recognition algorithms                                                    */
/*****************************************************************************/

bool is_ktree (graph&, int=2);

/*****************************************************************************/
/* Misc                                                                      */
/*****************************************************************************/

bool find_edge (graph&, edge&, const node&, const node&);
bool is_adjacent (graph&, const node&, const node&);
bool is_simplicial (graph&, node&);
bool is_clique (graph&, const list<node>&);

#endif
